"use strict";
/*
  csv-express
  Forked and modified by John J Czaplewski <jczaplew@gmail.com>
  Copyright 2011 Seiya Konno <nulltask@gmail.com>
  MIT Licensed

  https://github.com/jczaplew/csv-express/blob/master/lib/csv-express.js
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.csv = void 0;
// Configurable settings
const separator = ",";
const preventCast = false;
const ignoreNullOrUndefined = true;
// Stricter parseFloat to support hexadecimal strings from
// https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Global_Objects/parseFloat#A_stricter_parse_function
function filterFloat(value) {
    if (/^([-+])?([0-9]+(\.[0-9]+)?|Infinity)$/.test(value)) {
        return Number(value);
    }
    return NaN;
}
function escape(field) {
    if (ignoreNullOrUndefined && field == undefined) {
        return "";
    }
    if (preventCast) {
        return '="' + String(field).replace(/"/g, '""') + '"';
    }
    if (!isNaN(filterFloat(field)) && isFinite(field)) {
        return parseFloat(field);
    }
    return '"' + String(field).replace(/"/g, '""') + '"';
}
function csv(data, headerRow) {
    let body = "";
    // Append the header row to the response if requested
    if (headerRow) {
        body = headerRow.join(separator) + "\r\n";
    }
    // Convert the data to a CSV-like structure
    for (let i = 0; i < data.length; i++) {
        body += data[i].map(escape).join(separator) + "\r\n";
    }
    return body;
}
exports.csv = csv;
