"use strict";
// Get a sort_weight suitable for adding to top of list
// Original idea https://holtwick.de/en/blog/smart-table-reordering
Object.defineProperty(exports, "__esModule", { value: true });
exports.sortedItems = exports.moveSortWeight = exports.endSortWeight = exports.startSortWeight = void 0;
function startSortWeight(items) {
    return (items.reduce((acc, item) => Math.min(acc, item.sort_weight || 0), 0) -
        1 -
        Math.random());
}
exports.startSortWeight = startSortWeight;
// Get a sort_weight suitable for adding to end of list
function endSortWeight(items) {
    return (items.reduce((acc, item) => Math.max(acc, item.sort_weight || 0), 0) +
        1 +
        Math.random());
}
exports.endSortWeight = endSortWeight;
// The real magic
function moveSortWeight(newIndex, oldIndex, items) {
    let count = items.length;
    const moveLower = newIndex < oldIndex;
    if (count <= 0 || newIndex >= count - 1) {
        return endSortWeight(items);
    }
    if (newIndex <= 0) {
        return startSortWeight(items);
    }
    const step = moveLower ? -1 : 0;
    const lower = items[newIndex + step].sort_weight || 0;
    const upper = items[newIndex + step + 1].sort_weight || 0;
    const distance = upper - lower;
    if (distance === 0) {
        // Ugly list with no presets, make the best guess
        if (moveLower) {
            return startSortWeight(items);
        }
        return endSortWeight(items);
    }
    const middle = lower + distance / 2;
    const fuzzy = distance * 0.01 * (Math.random() - 0.5); // 1% fuzziness to avoid conflicts
    return middle + fuzzy;
}
exports.moveSortWeight = moveSortWeight;
function sortedItems(items) {
    items.sort((a, b) => (a.sort_weight || 0) - (b.sort_weight || 0));
    return items;
}
exports.sortedItems = sortedItems;
