"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoggerNodeHandler = void 0;
const log_js_1 = require("../common/log.js");
const time_js_1 = require("../common/time.js");
const tty_1 = __importDefault(require("tty"));
const colors = [6, 2, 3, 4, 5, 1];
function selectColor(namespace) {
    let hash = 0;
    for (let i = 0; i < namespace.length; i++) {
        hash = (hash << 5) - hash + namespace.charCodeAt(i);
        hash |= 0; // Convert to 32bit integer
    }
    return colors[Math.abs(hash) % colors.length];
}
let namespaces = {};
let time = time_js_1.getTimestamp();
const useColors = tty_1.default.isatty(process.stderr.fd);
function log(...args) {
    return process.stderr.write(args.join(" ") + "\n");
}
function LoggerNodeHandler(level = log_js_1.LogLevel.debug, opt = {
    colors: true,
    levelHelper: true,
    nameBrackets: false,
    padding: 16,
    // paddingLeft: 16,
}) {
    return (msg) => {
        if (msg.level < level)
            return;
        const timeNow = time_js_1.getTimestamp();
        let name = msg.name || "";
        let ninfo = namespaces[name || ""];
        if (ninfo == null) {
            ninfo = {
                color: selectColor(name),
                // time: timeNow
            };
            namespaces[name] = ninfo;
        }
        const diff = time_js_1.formatMilliseconds(timeNow - time);
        let args;
        let displayName = opt.nameBrackets ? `[${name}]` : name;
        if (opt.padding > 0) {
            displayName = displayName.padStart(opt.padding, " ");
            // displayName = displayName.padEnd(opt.padding, " ")
        }
        if (opt.colors && useColors) {
            const c = ninfo.color;
            const colorCode = "\u001B[3" + (c < 8 ? c : "8;5;" + c) + "m"; // ";1m "
            args = [`${colorCode}${displayName}\u001B[0m | `]; // opt.nameBrackets ? [`%c[${name}]`] : [`%c${name}`]
            args.push(...msg.messages);
            args.push(`${colorCode}+${diff}\u001B[0m`);
        }
        else {
            args = [displayName, ...msg.messages];
            args.push(`+${diff}`);
        }
        switch (msg.level) {
            case log_js_1.LogLevel.info:
                if (opt.levelHelper)
                    args[0] = `I|*   ` + args[0];
                log(...args);
                break;
            case log_js_1.LogLevel.warn:
                if (opt.levelHelper)
                    args[0] = `W|**  ` + args[0];
                log(...args);
                break;
            case log_js_1.LogLevel.error:
                if (opt.levelHelper)
                    args[0] = `E|*** ` + args[0];
                log(...args);
                break;
            default:
                if (opt.levelHelper)
                    args[0] = `D|    ` + args[0];
                log(...args);
                break;
        }
    };
}
exports.LoggerNodeHandler = LoggerNodeHandler;
