export declare enum LogLevel {
    debug = 0,
    info = 1,
    warn = 2,
    error = 3,
    fatal = 4
}
export interface LogMessage {
    level: LogLevel;
    name: string;
    messages: any[];
    line?: number;
    file?: string;
    timestamp?: number;
}
export declare type LogHandler = (msg: LogMessage) => void;
export declare function LoggerConsoleHandler(level?: LogLevel): LogHandler;
export interface LoggerInterface {
    (...messages: any[]): void;
    active: boolean;
    level: LogLevel;
    debug(...messages: any[]): void;
    info(...messages: any[]): void;
    warn(...messages: any[]): void;
    error(...messages: any[]): void;
    assert(cond: any, ...messages: any[]): void;
    assertEqual(value: any, expected: any, ...args: any[]): void;
    assertNotEqual(value: any, expected: any, ...args: any[]): void;
    extend(prefix: string): LoggerInterface;
}
export interface LoggerFactoryInterface {
    (name?: string): LoggerInterface;
    registerHandler(handler: LogHandler): void;
    _reject: RegExp[];
    _accept: RegExp[];
    setFilter(namespaces: string): void;
    _isNamespaceAllowed(name: string): boolean;
    _prefix: string;
    setPrefix(prefix: string): void;
    setHandlers(handlers?: LogHandler[]): void;
    level: number;
    /** @deprecated */
    setLogLevel(level?: LogLevel): void;
    extend(prefix: string): LoggerInterface;
    setFactory(factory: (name?: string) => LoggerInterface): void;
}
export declare function LoggerFactory(prefix?: string, opt?: {
    handlers?: LogHandler[];
    level?: number;
    accept?: RegExp[];
    reject?: RegExp[];
}): LoggerFactoryInterface;
export declare const Logger: LoggerFactoryInterface;
